wikla.dot

Nodes into DOTs.

Nodes into DOTs.
raw docstring

Dotclj/sprotocol

To paint a graph you must first define node properties.

To paint a graph you must first define node properties.

dot-childrenclj/s

(dot-children this)

Children to be graphed.

Children to be graphed.

dot-colorclj/s

(dot-color this)

Color.

Color.

dot-labelclj/s

(dot-label this)

Node Description.

Node Description.

dot-shapeclj/s

(dot-shape this)

Desired Shape.

Desired Shape.
raw docstring

node->colorclj/s

(node->color a-node)

Eats node, returns color depending on flag.

Eats node, returns color depending on flag.
raw docstring

node->styleclj/s

(node->style a-node)

Eats node, returns DOT style with shape, color and label.

Eats node, returns DOT style with shape, color and label.
raw docstring

tree->dotclj

(tree->dot tree file-name)

Eats node, saves tree to DOT.

Eats node, saves tree to DOT.
raw docstring

tree->pngclj

(tree->png tree file-name)

Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)

Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
raw docstring

tree->strclj/s

(tree->str tree)

Eats node, returns DOT graph string.

Walks the tree Depth-First, numbers nodes and links them.

Eats node, returns DOT graph string.

Walks the tree Depth-First, numbers nodes and links them.
raw docstring

tree->svgclj

(tree->svg tree file-name)

Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)

Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries