wikla.pre

Pre-cooked nodes & convenience constructors.

Pre-cooked nodes & convenience constructors.
raw docstring

a:breakpointclj/s

(a:breakpoint break?)
(a:breakpoint designation break?)

Maps (break? stash) callable: true to RUNNING, false to SUCCESS.

Hint: Need a FAILURE instead? Decorate!

Maps (break? stash) callable: true to RUNNING, false to SUCCESS.

Hint: Need a FAILURE instead? Decorate!
raw docstring

a:exceptionclj/s

Predefined Action always throwing ExceptionInfo.

Predefined Action always throwing ExceptionInfo.
raw docstring

a:failureclj/s

Predefined Action always returning Failure.

Predefined Action always returning Failure.
raw docstring

a:runningclj/s

Predefined Action always returning Running.

Predefined Action always returning Running.
raw docstring

a:successclj/s

Predefined Action always returning Success.

Predefined Action always returning Success.
raw docstring

bool-to-statusclj/s

(bool-to-status pred)
(bool-to-status pred strict?)

Creates an Action that will tick-pred-and-go, runs (pred stash).

Creates an Action that will tick-pred-and-go, runs (pred stash).
raw docstring

breakpoint-tick-fnclj/s

(breakpoint-tick-fn node)
(breakpoint-tick-fn node stash)

If (break? stash) then RUNNING, else SUCCESS.

Hint: ignoring stash and querying external state is fair game.

If (break? stash) then RUNNING, else SUCCESS.

Hint: ignoring stash and querying external state is fair game.
raw docstring

c:failureclj/s

Predefined Condition always returning Failure.

Predefined Condition always returning Failure.
raw docstring

c:successclj/s

Predefined Condition always returning Success.

Predefined Condition always returning Success.
raw docstring

d:failureclj/s

(d:failure node)

Decorator that will try to get child executed and then return Failure.

Will signal Running if child is Running.

Decorator that will try to get child executed and then return Failure.

Will signal Running if child is Running.
raw docstring

d:failure-on-runningclj/s

(d:failure-on-running node)

Decorator that will try to get child executed and fail if it's Running.

Will pass on either Success or Failure without issue.

Decorator that will try to get child executed and fail if it's Running.

Will pass on either Success or Failure without issue.
raw docstring

d:inversionclj/s

(d:inversion node)

Decorator that will try to get child executed and then reverse the Result.

Will signal Running if child is Running.

Decorator that will try to get child executed and then reverse the Result.

Will signal Running if child is Running.
raw docstring

d:map-resultclj/s

(d:map-result properties node)

Decorator that will modify Result status based on provided switches.

Default set to legal status will be just that. Illegal value will result in exception being raised.

Decorator that will modify Result status based on provided switches.

Default set to legal status will be just that. Illegal value will result in
exception being raised.
raw docstring

d:ready-on-runningclj/s

(d:ready-on-running node)

Decorator that will try to get child executed and rerun if it's Running.

Will pass on either Success or Failure without issue.

Decorator that will try to get child executed and rerun if it's Running.

Will pass on either Success or Failure without issue.
raw docstring

d:successclj/s

(d:success node)

Decorator that will try to get child executed and then return Success.

Will signal Running if child is Running.

Decorator that will try to get child executed and then return Success.

Will signal Running if child is Running.
raw docstring

d:success-on-runningclj/s

(d:success-on-running node)

Decorator that will try to get child executed and succeed if it's Running.

Will pass on either Success or Failure without issue.

Decorator that will try to get child executed and succeed if it's Running.

Will pass on either Success or Failure without issue.
raw docstring

just-throwclj/s

(just-throw node)
(just-throw node stash)

During tick it will just throw ex-info with properties and stash.

During tick it will just throw ex-info with properties and stash.
raw docstring

run-or-defaultclj/s

(run-or-default node sub)

Will map given flag via parameters :switch mapping.

Will map given flag via parameters :switch mapping.
raw docstring

tick-and-goclj/s

(tick-and-go node)
(tick-and-go node stash)

Run provided ifn over stash, return result as stash with wanted to Status.

Called without a stash runs the ifn with no args, possibly for side effects.

Run provided ifn over stash, return result as stash with wanted to Status.

Called without a stash runs the ifn with no args, possibly for side effects.
raw docstring

tick-pred-and-goclj/s

(tick-pred-and-go node)
(tick-pred-and-go node stash)

Feed stash to pred, then return status based on the result.

If not strict (the default), then truth-y is Success and false-y is Failure.

If strict and pred returns neither true, nor false, then Running is returned.

Can be called without stash, same rules apply.

Feed stash to pred, then return status based on the result.

If not strict (the default), then truth-y is Success and false-y is Failure.

If strict and pred returns neither true, nor false, then Running is returned.

Can be called without stash, same rules apply.
raw docstring

to-failureclj/s

(to-failure ifn)

Creates an Action that will tick-and-go, then return Failure.

Creates an Action that will tick-and-go, then return Failure.
raw docstring

to-successclj/s

(to-success ifn)

Creates an Action that will tick-and-go, then return Success.

Creates an Action that will tick-and-go, then return Success.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries