Nodes into DOTs.
Nodes into DOTs.
To paint a graph you must first define node properties.
To paint a graph you must first define node properties.
(dot-children this)Children to be graphed.
Children to be graphed.
(dot-color this)Color.
Color.
(dot-label this)Node Description.
Node Description.
(dot-shape this)Desired Shape.
Desired Shape.
(node->color a-node)Eats node, returns color depending on flag.
Eats node, returns color depending on flag.
(node->style a-node)Eats node, returns DOT style with shape, color and label.
Eats node, returns DOT style with shape, color and label.
(tree->dot tree file-name)Eats node, saves tree to DOT.
Eats node, saves tree to DOT.
(tree->png tree file-name)Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
(tree->str tree)Eats node, returns DOT graph string.
Walks the tree Depth-First, numbers nodes and links them.
Eats node, returns DOT graph string. Walks the tree Depth-First, numbers nodes and links them.
(tree->svg tree file-name)Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |