wikla.dot

Nodes into DOTs.

Nodes into DOTs.
raw docstring

Dotclj/s≠protocol

To paint a graph you must first define node properties.

To paint a graph you must first define node properties.

dot-colorclj/s

(dot-color this)

Color.

Color.

dot-childrenclj/s

(dot-children this)

Children to be graphed.

Children to be graphed.

dot-shapeclj/s

(dot-shape this)

Desired Shape.

Desired Shape.

dot-labelclj/s

(dot-label this)

Node Description.

Node Description.
raw docstring

node->colorclj/s

(node->color a-node)

Inputs: [a-node :- schema:Dot] Returns: s/Str

Eats node, returns color depending on flag.

Inputs: [a-node :- schema:Dot]
Returns: s/Str

Eats node, returns color depending on flag.
raw docstring

node->styleclj/s

(node->style a-node)

Inputs: [a-node :- schema:Dot] Returns: s/Str

Eats node, returns DOT style with shape, color and label.

Inputs: [a-node :- schema:Dot]
Returns: s/Str

Eats node, returns DOT style with shape, color and label.
raw docstring

schema:Dotclj/s

Schema definition for Dot protocol.

Schema definition for Dot protocol.
raw docstring

tree->dotclj

(tree->dot tree file-name)

Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool

Eats node, saves tree to DOT.

Inputs: [tree :- schema:Dot file-name :- s/Str]
Returns: s/Bool

Eats node, saves tree to DOT.
raw docstring

tree->pngclj

(tree->png tree file-name)

Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool

Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)

Inputs: [tree :- schema:Dot file-name :- s/Str]
Returns: s/Bool

Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
raw docstring

tree->strclj/s

(tree->str tree)

Inputs: [tree :- schema:Dot] Returns: s/Str

Eats node, returns DOT graph string.

Walks the tree Depth-First, numbers nodes and links them..

Inputs: [tree :- schema:Dot]
Returns: s/Str

Eats node, returns DOT graph string.

Walks the tree Depth-First, numbers nodes and links them..
raw docstring

tree->svgclj

(tree->svg tree file-name)

Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool

Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)

Inputs: [tree :- schema:Dot file-name :- s/Str]
Returns: s/Bool

Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries