Nodes into DOTs.
Nodes into DOTs.
To paint a graph you must first define node properties.
To paint a graph you must first define node properties.
(dot-color this)Color.
Color.
(dot-children this)Children to be graphed.
Children to be graphed.
(dot-shape this)Desired Shape.
Desired Shape.
(dot-label this)Node Description.
Node Description.
(node->color a-node)Inputs: [a-node :- schema:Dot] Returns: s/Str
Eats node, returns color depending on flag.
Inputs: [a-node :- schema:Dot] Returns: s/Str Eats node, returns color depending on flag.
(node->style a-node)Inputs: [a-node :- schema:Dot] Returns: s/Str
Eats node, returns DOT style with shape, color and label.
Inputs: [a-node :- schema:Dot] Returns: s/Str Eats node, returns DOT style with shape, color and label.
Schema definition for Dot protocol.
Schema definition for Dot protocol.
(tree->dot tree file-name)Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool
Eats node, saves tree to DOT.
Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool Eats node, saves tree to DOT.
(tree->png tree file-name)Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool
Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool Eats node, saves tree to PNG. (Shells out to dot -- Clojure only.)
(tree->str tree)Inputs: [tree :- schema:Dot] Returns: s/Str
Eats node, returns DOT graph string.
Walks the tree Depth-First, numbers nodes and links them..
Inputs: [tree :- schema:Dot] Returns: s/Str Eats node, returns DOT graph string. Walks the tree Depth-First, numbers nodes and links them..
(tree->svg tree file-name)Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool
Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
Inputs: [tree :- schema:Dot file-name :- s/Str] Returns: s/Bool Eats node, saves tree to SVG. (Shells out to dot -- Clojure only.)
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |