wikla.kit

Tools that make your life easier while working with wikla.

Tools that make your life easier while working with wikla.
raw docstring

extract-treesclj/s

(extract-trees zipable)
(extract-trees pred zipable)

Extract trees from a node. Either via predicate or every subtree.

Extract trees from a node. Either via predicate or every subtree.
raw docstring

flatten-treeclj/s

(flatten-tree zipable)
(flatten-tree pred zipable)

Accept tree, return a sequence of: [depth label flag mark].

Works on Node, zipped Node and Result, via wikla.zip/zipped-node.

Accept tree, return a sequence of: [depth label flag mark].

Works on Node, zipped Node and Result, via wikla.zip/zipped-node.
raw docstring

rewindclj/s

(rewind root)

Rewinds a statefull tree to a runnable condition, touches every node.

Requires nodes implementing wikla.zip/Zip and wikla.kit/Rewindable.

Please note that nodes are rewinded in visit order and must be reachable via zipper access.

Using rewind does not affect internal retry counters.

Rewinds a statefull tree to a runnable condition, touches every node.

Requires nodes implementing wikla.zip/Zip and wikla.kit/Rewindable.

Please note that nodes are rewinded in visit order and must be reachable
via zipper access.

Using rewind does not affect internal retry counters.
raw docstring

Rewindableclj/sprotocol

rewind-nodeclj/s

(rewind-node node)

Returns node to runnable condition. Not the children!

Returns node to runnable condition. Not the children!

show-flatclj/s

(show-flat flat)

Print structure returned by flatten-tree.

Print structure returned by flatten-tree.
raw docstring

show-trailclj/s

(show-trail ex)

Accept ex-data from all-tick, print path to affected node.

Accept ex-data from all-tick, print path to affected node.
raw docstring

show-treeclj/s

(show-tree zipable)
(show-tree pred zipable)

Walk a tree from Node or Result, display labels and flags.

One Node per line. Indent equal to two spaces x distance from root.

Mark nodes that pred is true for with [***].

Walk a tree from Node or Result, display labels and flags.

One Node per line. Indent equal to two spaces x distance from root.

Mark nodes that pred is true for with [***].
raw docstring

step-untilclj/s

(step-until a-stepper pred)

Accept Stepper and predicate, step until done or (pred a-stepper) is true.

Accept Stepper and predicate, step until done or (pred a-stepper) is true.
raw docstring

zip-edit-treeclj/s

(zip-edit-tree visitor zipable)

Zips the tree executing visitor function on every zipped node (loc).

You should use clojure.zip functions on the loc, like clojure.zip/edit.

Calls clojure.zip/root on loc when clojure.zip/end? reached.

Zips the tree executing visitor function on every zipped node (loc).

You should use clojure.zip functions on the loc, like clojure.zip/edit.

Calls clojure.zip/root on loc when clojure.zip/end? reached.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries