Pre-cooked nodes & convenience constructors.
Pre-cooked nodes & convenience constructors.
(a:breakpoint break?)(a:breakpoint designation break?)Maps (break? stash) callable: true to RUNNING, false to SUCCESS.
Hint: Need a FAILURE instead? Decorate!
Maps (break? stash) callable: true to RUNNING, false to SUCCESS. Hint: Need a FAILURE instead? Decorate!
Predefined Action always throwing ExceptionInfo.
Predefined Action always throwing ExceptionInfo.
Predefined Action always returning Failure.
Predefined Action always returning Failure.
Predefined Action always returning Running.
Predefined Action always returning Running.
Predefined Action always returning Success.
Predefined Action always returning Success.
(bool-to-status pred)(bool-to-status pred strict?)Creates an Action that will tick-pred-and-go, runs (pred stash).
Creates an Action that will tick-pred-and-go, runs (pred stash).
(breakpoint-tick-fn node)(breakpoint-tick-fn node stash)If (break? stash) then RUNNING, else SUCCESS.
Hint: ignoring stash and querying external state is fair game.
If (break? stash) then RUNNING, else SUCCESS. Hint: ignoring stash and querying external state is fair game.
Predefined Condition always returning Failure.
Predefined Condition always returning Failure.
Predefined Condition always returning Success.
Predefined Condition always returning Success.
(d:failure node)Decorator that will try to get child executed and then return Failure.
Will signal Running if child is Running.
Decorator that will try to get child executed and then return Failure. Will signal Running if child is Running.
(d:failure-on-running node)Decorator that will try to get child executed and fail if it's Running.
Will pass on either Success or Failure without issue.
Decorator that will try to get child executed and fail if it's Running. Will pass on either Success or Failure without issue.
(d:inversion node)Decorator that will try to get child executed and then reverse the Result.
Will signal Running if child is Running.
Decorator that will try to get child executed and then reverse the Result. Will signal Running if child is Running.
(d:map-result properties node)Decorator that will modify Result status based on provided switches.
Default set to legal status will be just that. Illegal value will result in exception being raised.
Decorator that will modify Result status based on provided switches. Default set to legal status will be just that. Illegal value will result in exception being raised.
(d:ready-on-running node)Decorator that will try to get child executed and rerun if it's Running.
Will pass on either Success or Failure without issue.
Decorator that will try to get child executed and rerun if it's Running. Will pass on either Success or Failure without issue.
(d:success node)Decorator that will try to get child executed and then return Success.
Will signal Running if child is Running.
Decorator that will try to get child executed and then return Success. Will signal Running if child is Running.
(d:success-on-running node)Decorator that will try to get child executed and succeed if it's Running.
Will pass on either Success or Failure without issue.
Decorator that will try to get child executed and succeed if it's Running. Will pass on either Success or Failure without issue.
(just-throw node)(just-throw node stash)During tick it will just throw ex-info with properties and stash.
During tick it will just throw ex-info with properties and stash.
(run-or-default node sub)Will map given flag via parameters :switch mapping.
Will map given flag via parameters :switch mapping.
(tick-and-go node)(tick-and-go node stash)Run provided ifn over stash, return result as stash with wanted to Status.
Called without a stash runs the ifn with no args, possibly for side effects.
Run provided ifn over stash, return result as stash with wanted to Status. Called without a stash runs the ifn with no args, possibly for side effects.
(tick-pred-and-go node)(tick-pred-and-go node stash)Feed stash to pred, then return status based on the result.
If not strict (the default), then truth-y is Success and false-y is Failure.
If strict and pred returns neither true, nor false, then Running is returned.
Can be called without stash, same rules apply.
Feed stash to pred, then return status based on the result. If not strict (the default), then truth-y is Success and false-y is Failure. If strict and pred returns neither true, nor false, then Running is returned. Can be called without stash, same rules apply.
(to-failure ifn)Creates an Action that will tick-and-go, then return Failure.
Creates an Action that will tick-and-go, then return Failure.
(to-success ifn)Creates an Action that will tick-and-go, then return Success.
Creates an Action that will tick-and-go, then return Success.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |