Zippers over nodes!
One and only thing you need to turn built-in Nodes into zippers is the
zipper function provided by this namespace. To allow custom nodes implement
Zip protocol accordingly.
Protip: zip-make-node expects children to be in shape of zip-children result.
Zippers over nodes! One and only thing you need to turn built-in Nodes into zippers is the `zipper` function provided by this namespace. To allow custom nodes implement Zip protocol accordingly. Protip: zip-make-node expects children to be in shape of zip-children result.
(node-maker node children)Helper that will reprocess zip-children into a branch.
Helper that will reprocess zip-children into a branch.
(return-zipper node)Helper that zipps a Node.
Helper that zipps a Node.
(what-is-this-thing? a-thing)Try to see if a given thing is a zipped Node, Node or Status.
Try to see if a given thing is a zipped Node, Node or Status.
Zip it, zip it good!
Zip it, zip it good!
(zip-branch? node)Are these nodes made for walking?
Are these nodes made for walking?
(zip-children node)Node goes in, BRRRT, children come out out.
Node goes in, BRRRT, children come out out.
(zip-make-node node children)It puts the children into the node.
It puts the children into the node.
(zip-zipper node)Return a zipper.
Return a zipper.
Accept Node, Result or zipped Node, return zipped Node.
Accept Node, Result or zipped Node, return zipped Node.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |